@@ -33,20 +33,23 @@
 
 #include <glib/gi18n-lib.h>
 #include <cairo.h>
-#include <cairo-xlib.h>
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
+
+#ifdef HAVE_X11
+#include <cairo-xlib.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtkx.h>
 #include <X11/Xatom.h>
+#include "panel-plug-private.h"
+#endif
 
 #include "mate-panel-applet.h"
 #include "panel-applet-private.h"
 #include "mate-panel-applet-factory.h"
 #include "mate-panel-applet-marshal.h"
 #include "mate-panel-applet-enums.h"
-#include "panel-plug-private.h"
 
 #define MATE_PANEL_APPLET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET, MatePanelAppletPrivate))
 
@@ -449,6 +452,8 @@ mate_panel_applet_set_locked_down (MatePanelApplet *applet,
 	g_object_notify (G_OBJECT (applet), "locked-down");
 }
 
+#ifdef HAVE_X11
+
 static Atom _net_wm_window_type = None;
 static Atom _net_wm_window_type_dock = None;
 static Atom _net_active_window = None;
@@ -533,6 +538,8 @@ mate_panel_applet_find_toplevel_dock_window (MatePanelApplet *applet,
 	return None;
 }
 
+#endif // HAVE_X11
+
 /* This function
  *   1) Gets the window id of the panel that contains the applet
  *	using XQueryTree and XGetWindowProperty to find an ancestor
@@ -543,6 +550,7 @@ void
 mate_panel_applet_request_focus (MatePanelApplet	 *applet,
 			    guint32	  timestamp)
 {
+#ifdef HAVE_X11
 	GdkScreen  *screen;
 	GdkWindow  *root;
 	GdkDisplay *display;
@@ -551,6 +559,9 @@ mate_panel_applet_request_focus (MatePanelApplet	 *applet,
 	Window	    xroot;
 	XEvent	    xev;
 
+	if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+		return;
+
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
 	screen	= gtk_window_get_screen (GTK_WINDOW (applet->priv->plug));
@@ -582,6 +593,7 @@ mate_panel_applet_request_focus (MatePanelApplet	 *applet,
 		    xroot, False,
 		    SubstructureRedirectMask | SubstructureNotifyMask,
 		    &xev);
+#endif
 }
 
 static GtkAction *
@@ -823,85 +835,9 @@ container_has_focusable_child (GtkContainer *container)
 	return retval;
 }
 
-static void
-mate_panel_applet_position_menu (GtkMenu   *menu,
-			    int       *x,
-			    int       *y,
-			    gboolean  *push_in,
-			    GtkWidget *widget)
-{
-	MatePanelApplet    *applet;
-	GtkAllocation   allocation;
-	GtkRequisition  requisition;
-	GdkDevice      *device;
-	GdkScreen      *screen;
-	int             menu_x = 0;
-	int             menu_y = 0;
-	int             pointer_x;
-	int             pointer_y;
-
-	g_return_if_fail (PANEL_IS_APPLET (widget));
-
-	applet = MATE_PANEL_APPLET (widget);
-
-	screen = gtk_widget_get_screen (widget);
-	gtk_menu_set_screen (menu, screen);
-
-	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
-	gdk_window_get_origin (gtk_widget_get_window (widget),
-			       &menu_x, &menu_y);
-
-	device = gdk_seat_get_pointer (gdk_display_get_default_seat (gtk_widget_get_display (widget)));
-	gdk_window_get_device_position (gtk_widget_get_window (widget), device, &pointer_x, &pointer_y, NULL);
-
-	gtk_widget_get_allocation (widget, &allocation);
-
-	menu_x += allocation.x;
-	menu_y += allocation.y;
-
-	if (applet->priv->orient == MATE_PANEL_APPLET_ORIENT_UP ||
-	    applet->priv->orient == MATE_PANEL_APPLET_ORIENT_DOWN) {
-		if (gtk_widget_get_direction (GTK_WIDGET (menu)) != GTK_TEXT_DIR_RTL) {
-			if (pointer_x < allocation.width &&
-			    requisition.width < pointer_x)
-				menu_x += MIN (pointer_x,
-					       allocation.width - requisition.width);
-		} else {
-			menu_x += allocation.width - requisition.width;
-			if (pointer_x > 0 && pointer_x < allocation.width &&
-			    pointer_x < allocation.width - requisition.width) {
-				menu_x -= MIN (allocation.width - pointer_x,
-					       allocation.width - requisition.width);
-			}
-		}
-		menu_x = MIN (menu_x, WidthOfScreen (gdk_x11_screen_get_xscreen (screen)) - requisition.width);
-
-		if (menu_y > HeightOfScreen (gdk_x11_screen_get_xscreen (screen)) / 2)
-			menu_y -= requisition.height;
-		else
-			menu_y += allocation.height;
-	} else  {
-		if (pointer_y < allocation.height &&
-		    requisition.height < pointer_y)
-			menu_y += MIN (pointer_y, allocation.height - requisition.height);
-		menu_y = MIN (menu_y, HeightOfScreen (gdk_x11_screen_get_xscreen (screen)) - requisition.height);
-
-		if (menu_x > WidthOfScreen (gdk_x11_screen_get_xscreen (screen)) / 2)
-			menu_x -= requisition.width;
-		else
-			menu_x += allocation.width;
-
-	}
-
-	*x = menu_x;
-	*y = menu_y;
-	*push_in = FALSE;
-}
-
 static void
 mate_panel_applet_menu_popup (MatePanelApplet *applet,
-			 guint        button,
-			 guint32      time)
+                              GdkEvent    *event)
 {
 	GtkWidget *menu;
 
@@ -919,11 +855,11 @@ mate_panel_applet_menu_popup (MatePanelApplet *applet,
 	context = gtk_widget_get_style_context (GTK_WIDGET(toplevel));
 	gtk_style_context_add_class(context,"gnome-panel-menu-bar");
 	gtk_style_context_add_class(context,"mate-panel-menu-bar");
-	gtk_menu_popup (GTK_MENU (menu),
-			NULL, NULL,
-			(GtkMenuPositionFunc) mate_panel_applet_position_menu,
-			applet,
-			button, time);
+	gtk_menu_popup_at_widget (GTK_MENU (menu),
+	                          GTK_WIDGET (applet),
+	                          GDK_GRAVITY_NORTH_WEST,
+	                          GDK_GRAVITY_NORTH_WEST,
+	                          event);
 }
 
 static gboolean
@@ -947,10 +883,12 @@ static gboolean
 mate_panel_applet_button_event (MatePanelApplet      *applet,
 			   GdkEventButton *event)
 {
+#ifdef HAVE_X11
 	GtkWidget *widget;
 	GdkWindow *window;
 	GdkWindow *socket_window;
 	XEvent     xevent;
+	GdkDisplay *display;
 
 	if (!applet->priv->out_of_process)
 		return FALSE;
@@ -963,13 +901,16 @@ mate_panel_applet_button_event (MatePanelApplet      *applet,
 	window = gtk_widget_get_window (widget);
 	socket_window = gtk_plug_get_socket_window (GTK_PLUG (widget));
 
+	display = gdk_display_get_default ();
+
+	if (!GDK_IS_X11_DISPLAY (display))
+		return FALSE;
+
 	if (event->type == GDK_BUTTON_PRESS) {
-		GdkDisplay *display;
 		GdkSeat *seat;
 
 		xevent.xbutton.type = ButtonPress;
 
-		display = gdk_display_get_default ();
 		seat = gdk_display_get_default_seat (display);
 
 		/* X does an automatic pointer grab on button press
@@ -998,16 +939,19 @@ mate_panel_applet_button_event (MatePanelApplet      *applet,
 	xevent.xbutton.button      = event->button;
 	xevent.xbutton.same_screen = TRUE; /* FIXME ? */
 
-	gdk_error_trap_push ();
+	gdk_x11_display_error_trap_push (display);
 
 	XSendEvent (GDK_WINDOW_XDISPLAY (window),
 		    GDK_WINDOW_XID (socket_window),
 		    False, NoEventMask, &xevent);
 
-	gdk_flush ();
-	gdk_error_trap_pop_ignored ();
+	gdk_display_flush (display);
+	gdk_x11_display_error_trap_pop_ignored (display);
 
 	return TRUE;
+#else
+	return FALSE;
+#endif
 }
 
 static gboolean
@@ -1024,7 +968,7 @@ mate_panel_applet_button_press (GtkWidget      *widget,
 	}
 
 	if (event->button == 3) {
-		mate_panel_applet_menu_popup (applet, event->button, event->time);
+		mate_panel_applet_menu_popup (applet, (GdkEvent *) event);
 
 		return TRUE;
 	}
@@ -1042,9 +986,10 @@ mate_panel_applet_button_release (GtkWidget      *widget,
 }
 
 static gboolean
-mate_panel_applet_popup_menu (GtkWidget *widget)
+mate_panel_applet_key_press_event (GtkWidget   *widget,
+			      GdkEventKey *event)
 {
-	mate_panel_applet_menu_popup (MATE_PANEL_APPLET (widget), 3, GDK_CURRENT_TIME);
+	mate_panel_applet_menu_popup (MATE_PANEL_APPLET (widget), (GdkEvent *) event);
 
 	return TRUE;
 }
@@ -1054,21 +999,23 @@ mate_panel_applet_get_preferred_width (GtkWidget *widget,
 				       int       *minimum_width,
 				       int       *natural_width)
 {
-	MatePanelApplet *applet = MATE_PANEL_APPLET (widget);
-	gint scale;
-
 	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->get_preferred_width (widget,
 										minimum_width,
 										natural_width);
 
+#if !GTK_CHECK_VERSION (3, 23, 0)
+	MatePanelApplet *applet = MATE_PANEL_APPLET (widget);
 	if (applet->priv->out_of_process) {
-		/* Out-of-process applets end up scaled up doubly. We divide by the scale factor to ensure
+		/* Out-of-process applets end up scaled up doubly with GTK 3.22.
+		 * For these builds divide by the scale factor to ensure
 		 * they are back at their own intended size.
 		 */
+		gint scale;
 		scale = gtk_widget_get_scale_factor (widget);
 		*minimum_width /= scale;
 		*natural_width /= scale;
 	}
+#endif
 }
 
 static void
@@ -1076,21 +1023,23 @@ mate_panel_applet_get_preferred_height (GtkWidget *widget,
 					int       *minimum_height,
 					int       *natural_height)
 {
-	MatePanelApplet *applet = MATE_PANEL_APPLET (widget);
-	gint scale;
-
 	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->get_preferred_height (widget,
 										minimum_height,
 										natural_height);
 
+#if !GTK_CHECK_VERSION (3, 23, 0)
+	MatePanelApplet *applet = MATE_PANEL_APPLET (widget);
 	if (applet->priv->out_of_process) {
-		/* Out-of-process applets end up scaled up doubly. We divide by the scale factor to ensure
+		gint scale;
+		/* Out-of-process applets end up scaled up doubly with GTK 3.22.
+		 * For these builds divide by the scale factor to ensure
 		 * they are back at their own intended size.
 		 */
 		scale = gtk_widget_get_scale_factor (widget);
 		*minimum_height /= scale;
 		*natural_height /= scale;
 	}
+#endif
 }
 
 static GtkSizeRequestMode
@@ -1246,6 +1195,7 @@ mate_panel_applet_parse_color (const gchar *color_str,
 	return gdk_rgba_parse (color, color_str);
 }
 
+#ifdef HAVE_X11
 static gboolean
 mate_panel_applet_parse_pixmap_str (const char *str,
 			       Window          *xid,
@@ -1300,10 +1250,10 @@ mate_panel_applet_create_foreign_surface_for_display (GdkDisplay *display,
 	gint x, y;
 	guint width, height, border, depth;
 
-	gdk_error_trap_push ();
+	gdk_x11_display_error_trap_push (display);
 	result = XGetGeometry (GDK_DISPLAY_XDISPLAY (display), xid, &window,
 	                       &x, &y, &width, &height, &border, &depth);
-	gdk_error_trap_pop_ignored ();
+	gdk_x11_display_error_trap_pop_ignored (display);
 
 	if (result == 0)
 		return NULL;
@@ -1324,6 +1274,7 @@ mate_panel_applet_get_pattern_from_pixmap (MatePanelApplet *applet,
 	GdkWindow       *window;
 	int              width;
 	int              height;
+	GdkDisplay      *display;
 	cairo_t         *cr;
 	cairo_pattern_t *pattern;
 
@@ -1333,8 +1284,9 @@ mate_panel_applet_get_pattern_from_pixmap (MatePanelApplet *applet,
 		return NULL;
 
 	window = gtk_widget_get_window (GTK_WIDGET (applet));
+	display = gdk_window_get_display (window);
 
-	background = mate_panel_applet_create_foreign_surface_for_display (gdk_window_get_display (window),
+	background = mate_panel_applet_create_foreign_surface_for_display (display,
 									   gdk_window_get_visual (window),
 									   xid);
 
@@ -1352,12 +1304,12 @@ mate_panel_applet_get_pattern_from_pixmap (MatePanelApplet *applet,
 	                            CAIRO_CONTENT_COLOR_ALPHA,
 	                            width,
 	                            height);
-	gdk_error_trap_push ();
+	gdk_x11_display_error_trap_push (display);
 	cr = cairo_create (surface);
 	cairo_set_source_surface (cr, background, -x, -y);
 	cairo_rectangle (cr, 0, 0, width, height);
 	cairo_fill (cr);
-	gdk_error_trap_pop_ignored ();
+	gdk_x11_display_error_trap_pop_ignored (display);
 
 	cairo_surface_destroy (background);
 	pattern = NULL;
@@ -1371,6 +1323,7 @@ mate_panel_applet_get_pattern_from_pixmap (MatePanelApplet *applet,
 
 	return pattern;
 }
+#endif
 
 static MatePanelAppletBackgroundType
 mate_panel_applet_handle_background_string (MatePanelApplet  *applet,
@@ -1403,27 +1356,34 @@ mate_panel_applet_handle_background_string (MatePanelApplet  *applet,
 		retval = PANEL_COLOR_BACKGROUND;
 
 	} else if (elements [0] && !strcmp (elements [0], "pixmap")) {
-		Window pixmap_id;
-		int             x, y;
+#ifdef HAVE_X11
+		if (GDK_IS_X11_DISPLAY (gdk_display_get_default ())) {
+			Window pixmap_id;
+			int             x, y;
 
-		g_return_val_if_fail (pattern != NULL, PANEL_NO_BACKGROUND);
+			g_return_val_if_fail (pattern != NULL, PANEL_NO_BACKGROUND);
 
-		if (!mate_panel_applet_parse_pixmap_str (elements [1], &pixmap_id, &x, &y)) {
-			g_warning ("Incomplete '%s' background type received: %s",
-				   elements [0], elements [1]);
+			if (!mate_panel_applet_parse_pixmap_str (elements [1], &pixmap_id, &x, &y)) {
+				g_warning ("Incomplete '%s' background type received: %s",
+					elements [0], elements [1]);
 
-			g_strfreev (elements);
-			return PANEL_NO_BACKGROUND;
+				g_strfreev (elements);
+				return PANEL_NO_BACKGROUND;
+			}
+
+			*pattern = mate_panel_applet_get_pattern_from_pixmap (applet, pixmap_id, x, y);
+			if (!*pattern) {
+				g_warning ("Failed to get pattern %s", elements [1]);
+				g_strfreev (elements);
+				return PANEL_NO_BACKGROUND;
+			}
+
+			retval = PANEL_PIXMAP_BACKGROUND;
+		} else
+#endif
+		{ // not using X11
+			g_warning("Received pixmap background type, which is only supported on X11");
 		}
-
-		*pattern = mate_panel_applet_get_pattern_from_pixmap (applet, pixmap_id, x, y);
-		if (!*pattern) {
-			g_warning ("Failed to get pattern %s", elements [1]);
-			g_strfreev (elements);
-			return PANEL_NO_BACKGROUND;
-		}
-
-		retval = PANEL_PIXMAP_BACKGROUND;
 	} else
 		g_warning ("Unknown background type received");
 
@@ -1845,6 +1805,14 @@ mate_panel_applet_constructor (GType                  type,
 	if (!applet->priv->out_of_process)
 		return object;
 
+#ifdef HAVE_X11
+	if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+#endif
+	{ // not using X11
+		g_warning ("Requested construction of an out-of-process applet, which is only possible on X11");
+		return object;
+	}
+
 	applet->priv->plug = gtk_plug_new (0);
 
 	GdkScreen *screen = gtk_widget_get_screen(GTK_WIDGET(applet->priv->plug));
@@ -1901,7 +1869,8 @@ mate_panel_applet_class_init (MatePanelAppletClass *klass)
 	widget_class->size_allocate = mate_panel_applet_size_allocate;
 	widget_class->focus = mate_panel_applet_focus;
 	widget_class->realize = mate_panel_applet_realize;
-	widget_class->popup_menu = mate_panel_applet_popup_menu;
+	widget_class->key_press_event = mate_panel_applet_key_press_event;
+
 
 	gobject_class->finalize = mate_panel_applet_finalize;
 
@@ -2065,6 +2034,30 @@ GtkWidget* mate_panel_applet_new(void)
 	return GTK_WIDGET(applet);
 }
 
+static GdkEvent *
+button_press_event_new (MatePanelApplet *applet,
+                        guint        button,
+                        guint        time)
+{
+  GdkDisplay *display;
+  GdkSeat *seat;
+  GdkDevice *device;
+  GdkEvent *event;
+
+  display = gdk_display_get_default ();
+  seat = gdk_display_get_default_seat (display);
+  device = gdk_seat_get_pointer (seat);
+
+  event = gdk_event_new (GDK_BUTTON_PRESS);
+
+  event->button.time = time;
+  event->button.button = button;
+
+  gdk_event_set_device (event, device);
+
+  return event;
+}
+
 static void
 method_call_cb (GDBusConnection       *connection,
 		const gchar           *sender,
@@ -2076,13 +2069,17 @@ method_call_cb (GDBusConnection       *connection,
 		gpointer               user_data)
 {
 	MatePanelApplet *applet = MATE_PANEL_APPLET (user_data);
+	GdkEvent *event;
 
 	if (g_strcmp0 (method_name, "PopupMenu") == 0) {
 		guint button;
 		guint time;
 
 		g_variant_get (parameters, "(uu)", &button, &time);
-		mate_panel_applet_menu_popup (applet, button, time);
+
+		event = button_press_event_new (applet, button, time);
+		mate_panel_applet_menu_popup (applet, event);
+		gdk_event_free (event);
 
 		g_dbus_method_invocation_return_value (invocation, NULL);
 	}
@@ -2232,6 +2229,7 @@ static void mate_panel_applet_factory_main_finalized(gpointer data, GObject* obj
 	}
 }
 
+#ifdef HAVE_X11
 static int (*_x_error_func) (Display *, XErrorEvent *);
 
 static int
@@ -2280,6 +2278,7 @@ _mate_panel_applet_setup_x_error_handler (void)
 
 	_x_error_func = XSetErrorHandler (_x_error_handler);
 }
+#endif
 
 static int
 _mate_panel_applet_factory_main_internal (const gchar               *factory_id,
@@ -2296,9 +2295,20 @@ _mate_panel_applet_factory_main_internal (const gchar               *factory_id,
 	g_return_val_if_fail(callback != NULL, 1);
 	g_assert(g_type_is_a(applet_type, PANEL_TYPE_APPLET));
 
-	if (out_process)
-	{
+
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ())) {
+		/*Use this both in and out of process as the tray applet always uses GtkSocket
+		*to handle GtkStatusIcons whether the tray itself is built in or out of process
+		*/
 		_mate_panel_applet_setup_x_error_handler();
+	} else
+#endif
+	{ // not using X11
+		if (out_process) {
+			g_warning("Requested out-of-process applet, which is only supported on X11");
+			return 1;
+		}
 	}
 
 	closure = g_cclosure_new(G_CALLBACK(callback), user_data, NULL);
@@ -2361,7 +2371,6 @@ mate_panel_applet_factory_setup_in_process (const gchar               *factory_i
 						    callback, user_data);
 }
 
-
 /**
  * mate_panel_applet_set_background_widget:
  * @applet: a #PanelApplet.
@@ -2384,13 +2393,18 @@ guint32
 mate_panel_applet_get_xid (MatePanelApplet *applet,
 		      GdkScreen   *screen)
 {
+	// out_of_process should only be true on X11, so an extra runtime Wayland check is not needed
 	if (applet->priv->out_of_process == FALSE)
 		return 0;
 
+#ifdef HAVE_X11
 	gtk_window_set_screen (GTK_WINDOW (applet->priv->plug), screen);
 	gtk_widget_show (applet->priv->plug);
 
 	return gtk_plug_get_id (GTK_PLUG (applet->priv->plug));
+#else
+	return 0;
+#endif
 }
 
 const gchar *
